/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

/* Weather Cameras with QtScript */

function tr(s) { return s; }

/* ImageViewer is capable to show image based on url. It also shows
 * additional information in QTextBrowser */
function ImageViewer(parent) {
	QWidget.call(this, parent);
	
	/* We should remember who is the parent object */
	this.parent = parent;
	
	/* rgba defines opacity */
	this.styleSheet = "QWidget { background-color: rgba(49,100,199,55%); } ";
	
	this.locLabel = new QLabel(this);
	
	this.imageLabel = new QLabel(this);
	this.imageLabel.setFixedSize(260,200);
	
	this.textBrowser = new QTextBrowser(this);
	this.textBrowser.setFixedSize(260,160);
	
	this.minimizeButton = new QPushButton(tr("Collapse"),this);
	this.minimizeButton["clicked"].connect(this, this.showMe);

	var layout = new QVBoxLayout(this);
	layout.addWidget(this.locLabel,0,0);
	layout.addWidget(this.imageLabel, 0, 0);
	layout.addWidget(this.textBrowser,0,0);
	layout.addStretch();
	layout.addWidget(this.minimizeButton,0,0);
	
	this.setLayout(layout);
	
	/* Set minimum size of the widget */
	this.minimumSize = new QSize(278,460);
	/* Widget shows after image is loaded */
	this.hide();
}

ImageViewer.prototype = new QWidget();

ImageViewer.prototype.showMe = function(showMe) {
	if(this.minimizeButton.text == "Expand" || showMe == true) {
		this.minimizeButton.text = tr("Collapse");
		this.resize(278,460);
		this.minimumSize = new QSize(278,460);
		this.locLabel.show();
		this.imageLabel.show();
		this.textBrowser.show();
	}
	else {
		this.locLabel.hide();
		this.imageLabel.hide();
		this.textBrowser.hide();
		this.minimumSize = new QSize(278,50);
		this.resize(278,50);
		this.minimizeButton.text = tr("Expand");
	}
}

/* Set QTextBrowser inner html */
ImageViewer.prototype.setBrowserHtml = function(html) {
	this.textBrowser.clear();
	html = '<font color="white">' + html + '</font>';
	this.textBrowser.insertHtml(html);
}

/* Set location label text */
ImageViewer.prototype.setLocLabel = function(text) {
	this.locLabel.text = '<font color="white">' + text + "</font>";
	//this.locLabel.show();
}

/* Load image to the widget */
ImageViewer.prototype.loadImage = function(url) {
	this.imageLabel.text = '<font color="white"><center>Loading image...</center></font>';
	accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, this.imageReplyFinished);
	accessManager.get(new QNetworkRequest((new QUrl(url))));
}

/* After image is succesfully loaded, scale it and show it */
ImageViewer.prototype.imageReplyFinished = function(reply) {
	var data = reply.readAll();
	var imagePixmap = new QPixmap();
	imagePixmap.loadFromData(data, 0, Qt.AutoColor);
	if(!imagePixmap.isNull()) {
		imagePixmap = imagePixmap.scaledToWidth(260,Qt.SmoothTransformation);
		this.imageLabel.setPixmap(imagePixmap);
	}
	else {
		/* imagePixmap was null, image data was not available (bad url)*/
		this.imageLabel.text = '<font color="white"><center>No image available</center></font>';
	}
	
	/* Show widget and try to resize it.
	 * Resizing may fail if there is no parent object */
	this.show();
	this.showMe(true);
	this.resize(278,460);

}

/* Main application widget */
/* Shows Google Map and uses Google Maps API to get directions
 * between two user clicked points. Seeks trough finnish road weather
 * cameras (inserted to the map as markers) and checks if those are
 * near estimated route. User can click markers and see image data
 * loaded from Finnish Road Services aka. Tiehallinto */
function QWeatherCameras(parent) {
	QWidget.call(this, parent);
	
	// Application is disabled until webview is fully loaded
	this.setDisabled(true);
	
	// Google Maps API key
	this.MY_API_KEY = "ABQIAAAAHHRHHBTb22UwAyFH2Mxh1hRH7ofs1HkLFKTfpNPzrQRiMcIbMhTcQOY43SyMyeX0pkYdb5KymD4USg";

	// QWebView shows a web page with full screen Google Map
	this.view = new QWebView(this);
	//this.view.parent = this;
	
	// let's do some initialization after view has loaded its web page
	this.view["loadFinished(bool)"].connect(this, this.loadFinished);
	
	// Use imageViewer widget to show results
	this.image = new ImageViewer(this);
	this.image.pos = new QPoint(5,5);
	
	// Layout is needed to make resizing work properly
	this.layout = new QVBoxLayout(this);
	this.layout.addWidget(this.view,0,0);
	this.layout.setContentsMargins(0,0,0,0);
	
	this.lastCameraUrl = undefined;
	this.lastCameraLoc = undefined;
	
	// Use timer to check if user has clicked a marker or not
	this.cameraTimer = new QTimer(this)
	this.cameraTimer["timeout"].connect(this, this.timerTimeout);
	
	// Try to load a web page with a Google Map
	try {
		this.view.load(new QUrl("http://www.cs.tut.fi/~salmin39/map/index.html"));
	} catch(err) {
		var messageBox = new QMessageBox(this);
		messageBox.text = "Error: " + err;
		messageBox.show();
	}
	
	this.windowTitle = tr("Finnish Road Weather Cameras");
	this.resize(800,480);
}

QWeatherCameras.prototype = new QWidget();



/* After the web page has been fully loaded we can do some
 * initialization and other stuff. If this is done too soon it will
 * not work at all */
QWeatherCameras.prototype.loadFinished = function(ok) {	
	/* Let's load quite a lot of stuff from external file. */
	var file = new QFile("./WeatherCamerasBrowserCode.js");	
	if(file.exists()) {
		file.open(QIODevice.ReadOnly);
		var stream = new QTextStream(file);
		while(!stream.atEnd()) {
			var line = line + stream.readLine() + "\n";
		}
		file.close();
		this.view.page().mainFrame().evaluateJavaScript(line);
	}
	else {
		var messageBox = new QMessageBox(this);
		messageBox.text = 'File "WeatherCamerasBrowserCode.js" not found. \nPlease run qs_eval from same directory with application.';
		messageBox.show();
	}
	
	// Starting to poll user clicked markers
	this.cameraTimer.start(1000);
	
	// Enable whole application
	this.setEnabled(true);
}

/* Function used to get the marker user clicked last */
QWeatherCameras.prototype.getUserLastClickedMarker = function() {
	return this.view.page().mainFrame().evaluateJavaScript("userClickedMarker;");
}

/* If user has clicked a marker we should show marker related weather
 * information if it is available */
QWeatherCameras.prototype.timerTimeout = function() {
	// get the marker
	var marker = this.getUserLastClickedMarker();
	this.view.page().mainFrame().evaluateJavaScript("userClickedMarker = undefined;");
	if(marker != undefined && marker.cameraUrl != undefined) {
		try {
			if(this.lastCameraUrl != marker.cameraUrl) {
				this.getWeatherCameraPage(marker.cameraUrl);
				this.lastCameraUrl = marker.cameraUrl;
			}
		}
		catch(err) {
			print(err);
		}
	}
}

/* Load a web page from tiehallinto servers */
QWeatherCameras.prototype.getWeatherCameraPage = function(url) {
	accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, this.weatherReplyFinished);
	accessManager.get(new QNetworkRequest(new QUrl(url)));
}

/* Parse and modify page we just loaded. Because Tiehallinto does not
 * have a proper API, we must do some quite ugly searching and replacing.
 * Hopefully they don't change their pages often... */
QWeatherCameras.prototype.weatherReplyFinished = function(reply) {
	var replyString = reply.readAll().toString();
	var index1 = replyString.indexOf('<IMG');
	var index2 = replyString.indexOf('" BORDER=0 WIDTH=526 HEIGHT=400');
	var result = replyString.substring(index1, index2);
	result = result.replace('<IMG SRC="', 'http://alk.tiehallinto.fi/');

	// Display image in imageViewer widget
	this.image.show();
	this.image.loadImage(result);
	
	// Tell Google maps to zoom to virtual center point
	var scriptString = 'useVirtualCenter = true'
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
	
	// Parsing and replacing for weather information
	index1 = replyString.indexOf('<BR><FONT SIZE="+1"><B>');
	if(index1 != -1) {
		index2 = replyString.indexOf('</B></FONT><BR><BR></TD></TR>');
		result = replyString.substring(index1 + 23,index2);
		this.image.setLocLabel(result);
	
		index1 = replyString.indexOf('<CENTER>');
		index2 = replyString.indexOf('</CENTER>');
		result = replyString.substring(index1 + 8,index2);
		result = result.replace('WIDTH="530"', ' ');
		result = result.replace('WIDTH="528"', ' ');
		result = result.replace('BGCOLOR="#C3DCE3"', ' ');
		result = result.replace('COLOR="#004E61"', 'COLOR="#F2F2F2"');
		for(i=0;i<8;i=i+1) {
			result = result.replace('COLOR="#004e61"', 'COLOR="#E6E6E6"');
			result = result.replace('<br>', ' ');
		}
	}
	else {
		result = "<center>No weather data<br>available</center>";
	}
	
	// Show weather information in imageViewer widget
	this.image.setBrowserHtml(result);
}

QByteArray.prototype.toString = function()
{
   ts = new QTextStream( this, QIODevice.ReadOnly );
   var codec = QTextCodec.codecForName(new QByteArray("ISO-8859-1"));
   ts.setCodec(codec);
   return ts.readAll();
}


/* To use the application in standalone mode, remove the comment around this code
var map = new QWeatherCameras();
map.show();

QCoreApplication.exec();
*/

